<?php

use CRM_Simpleorders_ExtensionUtil as E;

/**
 * Form controller class
 *
 * @see https://docs.civicrm.org/dev/en/latest/framework/quickform/
 */
class CRM_Simpleorders_Form_SoOrder extends CRM_Core_Form {

  private int $_soOrderId;
  private int $_contactId;
  private array $_currentData = [];

  /**
   * Overridden method to build form
   *
   * @return void
   * @throws CRM_Core_Exception
   */
  public function buildQuickForm(): void {
    $service = simpleorders_get_service();
    $this->add('hidden', 'so_order_id');
    if ($this->_action == CRM_Core_Action::VIEW) {
      $this->buildViewElements();
    }
    else {
      $this->addEntityRef('campaign_id', E::ts("Campaign"), [
        'entity' => 'campaign',
        'multiple' => FALSE,
        'api' => ['params' => ['is_active' => TRUE]],
        'placeholder' => E::ts('- select campaign -'),
        'select' => ['minimumInputLength' => 2],
      ]);
      $this->addContactElements();
      $this->add('datepicker', 'order_date', E::ts("Order Date"), [], TRUE, ['time' => FALSE]);
      $this->add('text', 'delivery_address', E::ts("Delivery Address"), ['size' => CRM_Utils_Type::HUGE], TRUE);
      $this->add('text', 'postcode', E::ts("Postcode"), [], TRUE);
      $this->add('text', 'city', E::ts("City"), [], TRUE);
      $this->addStateAndCountry();
      $this->add('text', 'source', E::ts("Source of the Order"));
      $this->add('checkbox', 'forwarded', E::ts('Order forwarded?'));
      $this->addButtons([
        ['type' => 'next', 'name' => E::ts('Save'), 'isDefault' => TRUE],
        ['type' => 'submit', 'name' => E::ts('Save and Order Items')],
        ['type' => 'cancel', 'name' => E::ts('Cancel')],
      ]);
    }
    // export form elements
    $this->assign('elementNames', $service->getRenderableElementNames($this));
    parent::buildQuickForm();
  }
  private function buildViewElements() {
    $service = simpleorders_get_service();
    $this->add('text', 'campaign_id', E::ts("Campaign"), [
      'size' => CRM_Utils_Type::HUGE,
      'readonly' => TRUE]);
    $this->add('text', $service->getFirstContactName() . "_contact_id", $service->getFirstContactLabel(), ['readonly' => TRUE]);
    $this->add('text', $service->getSecondContactName() . "_contact_id", $service->getSecondContactLabel(), ['readonly' => TRUE]);
    $this->add('text', 'order_date', E::ts("Order Date"), ['readonly' => TRUE]);
    $this->add('text', 'delivery_address', E::ts("Delivery Address"), [
      'size' => CRM_Utils_Type::HUGE,
      'readonly' => TRUE]);
    $this->add('text', 'postcode', E::ts("Postcode"), ['readonly' => TRUE]);
    $this->add('text', 'city', E::ts("City"), [
      'size' => CRM_Utils_Type::HUGE,
      'readonly' => TRUE]);
    if ($service->usesState()) {
      $this->add('text', 'state_id', E::ts("State/Province"), ['readonly' => TRUE]);
    }
    if ($service->usesCountry()) {
      $this->add('text', 'country_id', E::ts("County"), ['readonly' => TRUE]);
    }
    $this->add('text', 'source', E::ts("Source of the Order"), ['readonly' => TRUE]);
    $this->add('text', 'forwarded', E::ts("Order forwarded?"), ['readonly' => TRUE]);
    $this->addButtons([
      ['type' => 'next', 'name' => E::ts('Done'), 'isDefault' => TRUE],
      ['type' => 'cancel', 'name' => E::ts('Cancel')],
    ]);
  }

  /**
   * Method to add the state/province and country elements if needed
   *
   * @return void
   */
  private function addStateAndCountry(): void {
    $service = simpleorders_get_service();
    if ($service->usesState()) {
      $this->addEntityRef('state_id', E::ts("State/Province"), [
        'entity' => 'state_province',
        'multiple' => FALSE,
        'api' => [
          'params' => [
            'is_active' => TRUE,
          ]
        ]]);
    }
    if ($service->usesCountry()) {
      $this->addEntityRef('country_id', E::ts("Country"), [
        'entity' => 'country',
        'multiple' => FALSE,
        'api' => [
          'params' => [
            'is_active' => TRUE,
          ]
        ]]);
    }
  }

  /**
   * Method to add the contact elements, based on the names and labels from the
   * settings for the extension
   *
   * @return void
   */
  private function addContactElements(): void {
    $service= simpleorders_get_service();
    if ($this->_action == CRM_Core_Action::ADD && !isset($this->_contactId)) {
      $this->addEntityRef($service->getFirstContactName() . '_contact_id', $service->getFirstContactLabel(), [
        'api' => ['params' => ['is_deceased' => FALSE]]
      ], TRUE);
      $this->addEntityRef($service->getSecondContactName() . '_contact_id', $service->getSecondContactLabel(), [
        'api' => ['params' => ['is_deceased' => FALSE]]
      ], TRUE);
    }
    else {
      $this->add('hidden', 'contact_id');
      $this->addEntityRef($service->getFirstContactName() . "_contact_id", $service->getFirstContactLabel(), [
        'placeholder' => E::ts('- select contact -'),
        'select' => ['minimumInputLength' => 3],
      ]);
      $this->addEntityRef($service->getSecondContactName() . "_contact_id", $service->getSecondContactLabel(), [
        'placeholder' => E::ts('- select contact -'),
        'select' => ['minimumInputLength' => 3],
      ]);
    }
  }

  /**
   * Overridden method to prepare form
   *
   * @return void
   * @throws CRM_Core_Exception
   */
  public function preProcess(): void {
    $contactId = CRM_Utils_Request::retrieveValue('cid', 'Integer');
    if (!$contactId) {
      $contactId = $this->getSubmitValue('contact_id');
    }
    if ($contactId) {
      $this->_contactId = $contactId;
    }
    if ($this->_action == CRM_Core_Action::ADD) {
      if ($contactId) {
        $service = simpleorders_get_service();
        CRM_Utils_System::setTitle(E::ts("Add Order for ") . $service->getContactName($this->_contactId));
      }
      else {
        CRM_Utils_System::setTitle(E::ts("Add Order"));
      }
    }
    else {
      $service = simpleorders_get_service();
      $soOrderId = CRM_Utils_Request::retrieveValue('oid', 'Integer');
      if (!$soOrderId) {
        $soOrderId = $this->getSubmittedValue('so_order_id');
      }
      if ($soOrderId) {
        if ($this->_action == CRM_Core_Action::UPDATE) {
          CRM_Utils_System::setTitle(E::ts("Update Order ") . $service->getOrderTitleForForm($soOrderId));
        }
        if ($this->_action == CRM_Core_Action::VIEW) {
          CRM_Utils_System::setTitle(E::ts("View Order ") . $service->getOrderTitleForForm($soOrderId));
        }
        $this->_soOrderId = $soOrderId;
        $soOrder = new CRM_Simpleorders_BAO_SoOrder();
        $this->_currentData = $soOrder->getSoOrderWithOrderContacts($this->_soOrderId);
      }
    }
    if ($contactId) {
      CRM_Core_Session::singleton()->pushUserContext(CRM_Utils_System::url('civicrm/contact/view', [
        'reset' => TRUE,
        'force' => TRUE,
        'selectedChild' => 'contact_so_orders',
        'cid' => $this->_contactId]));
    }
  }

  /**
   * Overridden parent method to set defaults
   *
   * @return array
   */
  public function setDefaultValues(): array {
    $defaults = [];
    if (isset($this->_contactId)) {
      $defaults = ['contact_id' => $this->_contactId];
    }
    if ($this->_action == CRM_Core_Action::UPDATE || $this->_action == CRM_Core_Action::VIEW) {
      $defaults['so_order_id'] = $this->_soOrderId;
      foreach ($this->_currentData as $currentField => $currentValue) {
        $defaults[$currentField] = $currentValue;
      }
      if ($this->_action == CRM_Core_Action::VIEW) {
        $this->setViewDefaults($defaults);
      }
      $service =  simpleorders_get_service();
      $service->setContactDefaults($defaults, $this->_currentData);
    }
    return $defaults;
  }

  /**
   * Method to set the defaults specific to the view mode
   *
   * @param array $defaults
   * @return void
   */
  private function setViewDefaults(array &$defaults) {
    $service = simpleorders_get_service();
    if ($service->usesState()) {
      $defaults['state_id'] = $service->getStateName($this->_currentData['state_id']);
    }
    if ($service->usesCountry()) {
      $defaults['country_id'] = $service->getCountryName($this->_currentData['country_id']);
    }
    $defaults['campaign_id'] = $service->getCampaignTitle($this->_currentData['campaign_id']);
    if ($defaults['forwarded']) {
      $defaults['forwarded'] = E::ts("Yes");
    }
    else {
      $defaults['forwarded'] = E::ts("No");
    }
    $this->assign('so_order_items', $this->getViewOrderItems());
  }

  /**
   * Method to get the items for the view action
   *
   * @return array
   */
  private function getViewOrderItems(): array {
    $orderItems = [];
    try {
      $soOrderItems = \Civi\Api4\SoOrderItem::get(FALSE)
        ->addSelect('quantity', 'item_id.name', 'item_id.item_type:label')
        ->addWhere('order_id', '=', $this->_soOrderId)
        ->execute();
      foreach ($soOrderItems as $soOrderItem) {
        $orderItems[] = [
          'item' => $soOrderItem['item_id.name'],
          'item_type' => $soOrderItem['item_id.item_type:label'],
          'quantity' => $soOrderItem['quantity'],
        ];
      }
    }
    catch (\CRM_Core_Exception $ex) {
    }
    return $orderItems;
  }


  /**
   * Overridden parent method to process form submission
   *
   * @return void
   */
  public function postProcess(): void {
    $submitValues = $this->getSubmittedValues();
    if ($submitValues['contact_id']) {
      $this->_contactId = $submitValues['contact_id'];
    }
    if ($submitValues['so_order_id']) {
      $this->_soOrderId = $submitValues['so_order_id'];
    }
    $soOrder = new CRM_Simpleorders_BAO_SoOrder();
    if ($this->_action == CRM_Core_Action::ADD) {
      $this->_soOrderId = $soOrder->createSoOrder($submitValues);
      CRM_Core_Session::setStatus(E::ts("Order saved"), E::ts("Saved"), "success");
    }
    if ($this->_action == CRM_Core_Action::UPDATE) {
      $soOrder->updateSoOrder($this->_soOrderId, $submitValues);
    }
    $this->processItemsIfNeeded();
    parent::postProcess();
  }

  /**
   * Method to process to the order item form or page if needed
   *
   * @return void
   */
  private function processItemsIfNeeded(): void {
    if ($this->controller->getButtonName() == "_qf_SoOrder_submit") {
      if ($this->_action == CRM_Core_Action::ADD) {
        CRM_Utils_System::redirect(CRM_Utils_System::url('civicrm/simpleorders/form/soorderitem', [
          'reset' => TRUE,
          'action' => 'add',
          'oid' => $this->_soOrderId,
        ]));
      }
      if ($this->_action == CRM_Core_Action::UPDATE) {
        CRM_Utils_System::redirect(CRM_Utils_System::url('civicrm/simpleorders/page/soorderitem', [
          'reset' => TRUE,
          'oid' => $this->_soOrderId,
        ]));
      }
    }
  }

}
