<?php
use CRM_Simpleorders_ExtensionUtil as E;

/**
 * Class to show order for a contact (tab on contact summary)
 */
class CRM_Simpleorders_Page_SoContactOrder extends CRM_Core_Page {
  private $_contactId;

  /**
   * Overridden parent method to run the page
   *
   * @return void
   * @throws CRM_Core_Exception
   */
  public function run() {
    $this->initializePage();
    $this->assign('so_orders', $this->getContactOrders());
    parent::run();
  }

  /**
   * Method to get all contact orders
   *
   * @return array
   */
  public function getContactOrders(): array {
    $contactOrders = [];
    try {
      $orders = \Civi\Api4\SoOrderContact::get(TRUE)
        ->addSelect('role', 'role:label', 'order_id.*', 'campaign.title')
        ->addWhere('contact_id', '=', $this->_contactId)
        ->addJoin('Campaign AS campaign', 'LEFT', ['order_id.campaign_id', '=', 'campaign.id'])
        ->execute();
      foreach ($orders as $order) {
        $contactOrders[$order['id']] = $this->setOrderValues($order);
      }
    }
    catch (CRM_Core_Exception $ex) {
    }
    return $contactOrders;
  }

  /**
   * Method to set the values for the order row
   *
   * @param array $order
   * @return array
   */
  private function setOrderValues(array $order): array {
    $contactOrderValues = [
      'role' => $order['role:label'],
      'campaign' => $order['campaign.title'],
      'order_date' => $order['order_id.order_date'],
      'delivery_address' => $order['order_id.delivery_address'],
      'postcode' => $order['order_id.postcode'],
      'city' => $order['order_id.city'],
      'source' => $order['order_id.source'],
      'actions' => $this->setActionLinks($order['order_id.id']),
    ];
    if ($order['order_id..forwarded']) {
      $contactOrderValues['forwarded'] = E::ts("Yes");
    }
    else {
      $contactOrderValues['forwarded'] = E::ts("No");
    }
    $otherContact = $this->getOtherContact($order['order_id.id'], $this->_contactId);
    if (isset($otherContact['id'])) {
      $contactOrderValues['other_contact_id'] = $otherContact['id'];
    }
    if (isset($otherContact['name'])) {
      $contactOrderValues['other_contact'] = $otherContact['name'];
    }
    return $contactOrderValues;
  }

  /**
   * Method to get the other contact for the order
   *
   * @param int $orderId
   * @param int $contactId
   * @return array
   */
  private function getOtherContact(int $orderId, int $contactId): array {
    $otherContact = [];
    try {
      $orderContact = \Civi\Api4\SoOrderContact::get(FALSE)
        ->addSelect('contact_id.display_name', 'contact_id', 'role:label')
        ->addWhere('order_id', '=', $orderId)
        ->addWhere('contact_id', '!=', $contactId)
        ->execute()->first();
      if (isset($orderContact['contact_id'])) {
        $otherContact = [
          'id' => $orderContact['contact_id'],
          'name' => $orderContact['contact_id.display_name'] . " (" . $orderContact['role:label'] . ")",
        ];
      }
    }
    catch (\CRM_Core_Exception $ex) {
    }
    return $otherContact;
  }

  /**
   * Method to set the action links
   *
   * @param int $orderId
   * @return array
   */
  private function setActionLinks(int $orderId): array {
    $deleteUrl = CRM_Utils_System::url('civicrm/simpleorders/form/delsoorder', [
      'reset' => TRUE,
      'action' => 'delete',
      'oid' => $orderId,
      'cid' => $this->_contactId,
    ],
    );
    $editUrl = CRM_Utils_System::url('civicrm/simpleorders/form/soorder', [
      'reset' => TRUE,
      'action' => 'update',
      'oid' => $orderId,
      'cid' => $this->_contactId,
    ]);
    $viewUrl = CRM_Utils_System::url('civicrm/simpleorders/form/soorder', [
      'reset' => TRUE,
      'action' => 'view',
      'oid' => $orderId,
      'cid' => $this->_contactId,
    ]);
    return [
      '<a class="action-item" title="View" href="' . $viewUrl . '">' . E::ts('View') . '</a>',
      '<a class="action-item" title="Edit" href="' . $editUrl . '">' . E::ts('Edit') . '</a>',
      '<a class="action-item" title="Delete" href="' . $deleteUrl . '">' . E::ts('Delete') . '</a>',
    ];
  }

  /**
   * Initialize the page
   *
   * @return void
   * @throws CRM_Core_Exception
   */
  private function initializePage(): void {
    $this->_contactId = CRM_Utils_Request::retrieveValue('id', 'Integer');
    $this->assign('new_url', CRM_Utils_System::url('civicrm/simpleorders/form/soorder', [
      'reset' => TRUE,
      'action' => 'add',
      'cid' => $this->_contactId,
    ]));
    $service = simpleorders_get_service();
    CRM_Utils_System::setTitle(E::ts("Orders for ") . $service->getContactName($this->_contactId));
  }

}
