<?php

namespace Civi\Simpleorders;
use CRM_Core_Action;
use CRM_Simpleorders_ExtensionUtil as E;

/**
 * Class for Simple Orders configuration and helper functions (with container usage)
 *
 * @author Erik Hommel (CiviCooP) <erik.hommel@civicoop.org>
 * @license AGPL-3.0
 */
class Service {

  /**
   * @var Service
   */
  protected static $singleton;

  /**
   * Service constructor.
   */
  public function __construct() {
    if (!self::$singleton) {
      self::$singleton = $this;
    }
  }

  /**
   * @return Service
   */
  public static function getInstance(): Service {
    if (!self::$singleton) {
      self::$singleton = new Service();
    }
    return self::$singleton;
  }

  /**
   * Method to get the name of a contact
   *
   * @param int $contactId
   * @return string
   */
  public function getContactName(int $contactId): string {
    $contactName = "";
    try {
      $contact = \Civi\Api4\Contact::get(FALSE)
        ->addWhere('id', '=', $contactId)
        ->addSelect('display_name')
        ->execute()->first();
      if (isset($contact['display_name'])) {
        $contactName = $contact['display_name'];
      }
    }
    catch (\CRM_Core_Exception $ex) {
    }
    return $contactName;
  }

  /**
   * @return bool
   */
  public function usesState(): bool {
    return \Civi::settings()->get('simpleorders_use_state');
  }

  /**
   * @return bool
   */
  public function usesCountry(): bool {
    return \Civi::settings()->get('simpleorders_use_country');
  }

  /**
   * @return string
   */
  public function getFirstContactName(): string {
    return \Civi::settings()->get('simpleorders_first_contact');;
  }

  /**
   * @return string
   */
  public function getFirstContactLabel(): string {
    $label = [];
    try {
      $optionValue = \Civi\Api4\OptionValue::get(FALSE)
        ->addSelect('label')
        ->addWhere('option_group_id:name', '=', 'so_contact_role')
        ->addWhere('value', '=', $this->getFirstContactName())
        ->execute()->first();
      if (isset($optionValue['label'])) {
        $label = $optionValue['label'];
      }
    }
    catch (\CRM_Core_Exception $ex) {
    }
    return $label;
  }

  /**
   * @return string
   */
  public function getSecondContactName(): string {
    return \Civi::settings()->get('simpleorders_second_contact');;
  }

  /**
   * @return string
   */
  public function getSecondContactLabel(): string {
    $label = [];
    try {
      $optionValue = \Civi\Api4\OptionValue::get(FALSE)
        ->addSelect('label')
        ->addWhere('option_group_id:name', '=', 'so_contact_role')
        ->addWhere('value', '=', $this->getSecondContactName())
        ->execute()->first();
      if (isset($optionValue['label'])) {
        $label = $optionValue['label'];
      }
    }
    catch (\CRM_Core_Exception $ex) {
    }
    return $label;
  }

  /**
   * Method to set part of the title for an order to use as form title
   *
   * @param int $soOrderId
   * @return string
   */
  public function getOrderTitleForForm(int $soOrderId): string {
    $soOrderContact = new \CRM_Simpleorders_BAO_SoOrderContact();
    $contacts = $soOrderContact->getContactsOnOrder($soOrderId);
    $firstContact = $this->getContactName($contacts[$this->getFirstContactName()]);
    $secondContact = $this->getContactName($contacts[$this->getSecondContactName()]);
    return $this->getFirstContactLabel() . " " . $firstContact . " " . $this->getSecondContactLabel() . " " . $secondContact;
  }

  /**
   * Method to set the defaults for the contacts on order
   *
   * @param array $defaults
   * @param array $currentData
   * @return void
   */
  public function setContactDefaults(array &$defaults, array $currentData): void {
    $service= simpleorders_get_service();
    if ($this->_action == CRM_COre_Action::VIEW) {
      if (isset($currentData[$service->getFirstContactName()])) {
        $defaults[$service->getFirstContactName() . "_contact_id"] = $currentData[$service->getFirstContactName()];
        unset($defaults[$service->getFirstContactName()]);
      }
      if (isset($currentData[$service->getSecondContactName()])) {
        $defaults[$service->getSecondContactName() . "_contact_id"] = $currentData[$service->getSecondContactName()];
        unset($defaults[$service->getSecondContactName()]);
      }
    }
    else {
      if (isset($currentData[$service->getFirstContactName()])) {
        $defaults[$service->getFirstContactName() . "_contact_id"] = $service->getContactName($currentData[$service->getFirstContactName()]);
      }
      if (isset($currentData[$service->getSecondContactName()])) {
        $defaults[$service->getSecondContactName() . "_contact_id"] = $service->getContactName($currentData[$service->getSecondContactName()]);
      }
    }
  }

  /**
   * @param int $stateId
   * @return string
   */
  public function getStateName(int $stateId): string {
    $name = [];
    try {
      $state = \Civi\Api4\StateProvince::get(FALSE)
        ->addSelect('name')
        ->addWhere('id', '=', $stateId)
        ->execute()->first();
      if (isset($state['name'])) {
        $name = $state['name'];
      }
    }
    catch (\CRM_Core_Exception $ex) {
    }
    return $name;
  }

  /**
   * @param int $countryId
   * @return string
   */
  public function getCountryName(int $countryId): string {
    $name = [];
    try {
      $country = \Civi\Api4\Country::get(FALSE)
        ->addSelect('name')
        ->addWhere('id', '=', $countryId)
        ->execute()->first();
      if (isset($country['name'])) {
        $name = $country['name'];
      }
    }
    catch (\CRM_Core_Exception $ex) {
    }
    return $name;
  }

  /**
   * Method to get the campaign title
   *
   * @param int $campaignId
   * @return string
   */
  public function getCampaignTitle(int $campaignId): string {
    $title = "";
    try {
      $campaign = \Civi\Api4\Campaign::get(FALSE)
        ->addSelect('title')
        ->addWhere('id', '=', $campaignId)
        ->execute()->first();
      if (isset($campaign['title'])) {
        $title = $campaign['title'];
      }
    }
    catch (\CRM_Core_Exception $ex) {
    }
    return $title;
  }


  /**
   * Get the fields/elements defined in this form.
   *
   * @param \CRM_Core_Form $form
   * @return array (string)
   */
  public function getRenderableElementNames(\CRM_Core_Form $form): array {
    // The _elements list includes some items which should not be
    // auto-rendered in the loop -- such as "qfKey" and "buttons".  These
    // items don't have labels.  We'll identify renderable by filtering on
    // the 'label'.
    $elementNames = [];
    foreach ($form->_elements as $element) {
      $label = $element->getLabel();
      if (!empty($label)) {
        $elementNames[] = $element->getName();
      }
    }
    return $elementNames;
  }

}
